package com.samsung.sbkpublicaddressappforblog;

import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import com.journeyapps.barcodescanner.BarcodeEncoder;
import com.samsung.android.sdk.coldwallet.ScwErrorCode;
import com.samsung.android.sdk.coldwallet.ScwService;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity {

    private ArrayList<String> hdPathList = new ArrayList<>();
    private Integer accountIndex = 0;
    private ArrayAdapter spinnerAdapter;
    private ScwService mScwService = ScwService.getInstance();
    private String selectedAddress;
    private String LOG_TAG = "KeystoreApp";

    //For Bitcoin you can use hpPathPrefix value "m/44'/0'/0'/0/"
    private String hpPathPrefix = "m/44'/60'/0'/0/";


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        //HD Path of the first Account
        hdPathList.add(hpPathPrefix + accountIndex);
        //Create Adapter for pubic address spinner
        spinnerAdapter = new ArrayAdapter(this, R.layout.support_simple_spinner_dropdown_item, new ArrayList<String>());
        Spinner publicAddressSpinner = findViewById(R.id.publicAddressSpinner);
        final TextView selectedAddressTextView = findViewById(R.id.selected_public_address_textview);
        final ImageView qrCodeImageView = findViewById(R.id.qrCodeImageView);
        publicAddressSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int position, long l) {
                //Get Selected Address from spinner
                selectedAddress = adapterView.getItemAtPosition(position).toString();
                //Display Selected Address in TextView
                selectedAddressTextView.setText(selectedAddress);
                //Display Selected address as QR
                qrCodeImageView.setImageBitmap(generateQRCode(selectedAddress));
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }
        });
        publicAddressSpinner.setAdapter(spinnerAdapter);
        getPublicAddress();
    }

    private void getPublicAddress() {
        ScwService.ScwGetAddressListCallback addressListCallback = new ScwService.ScwGetAddressListCallback() {
            @Override
            public void onSuccess(final List<String> publicAddressList) {
                //After Address List has been fetched Spinner is updated with new list
                //Runnable is use for running UI updating code in main thread
                //Change in list is notified, triggering UI update
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        //Clear existing list
                        spinnerAdapter.clear();
                        //New list is added
                        spinnerAdapter.addAll(publicAddressList);
                        spinnerAdapter.notifyDataSetChanged();
                        if (publicAddressList.size() == 1) {
                            showToast(publicAddressList.size() + " address fetched.");
                        } else {
                            showToast(publicAddressList.size() + " addresses fetched.");
                        }
                    }
                });
            }

            @Override
            public void onFailure(int errorCode) {
                switch (errorCode) {
                    case ScwErrorCode.ERROR_INVALID_SCW_APP_ID:
                        Log.e(LOG_TAG,"Developer option not enabled.");
                        break;
                    case ScwErrorCode.ERROR_CHECK_APP_VERSION_FAILED:
                        Log.e(LOG_TAG,"Check internet connection.");
                        break;
                    case ScwErrorCode.ERROR_OP_FAIL:
                        Log.e(LOG_TAG,"Operation Failed");
                        break;
                    default:
                        break;
                }
                Log.e(LOG_TAG,"Error with Error Code: "+errorCode);
            }
        };
        if (mScwService == null) {
            Log.e(LOG_TAG, "Keystore is not supported in this device.");
        } else {
            //If Keystore is supported on device address list is requested
            mScwService.getAddressList(addressListCallback, hdPathList);
        }
    }

    public void addAccount(View view) {
        //Account Index is incremented by 1 to get the new account
        accountIndex++;
        //HDPath of new account is added to hdPathList
        hdPathList.add(hpPathPrefix + accountIndex);
        showToast("HDPath Added to list");
        //Public Address of new account is fetched
        getPublicAddress();
    }

    public void copyAddress(View view) {
        ClipboardManager clipboardManager = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
        ClipData clipData = ClipData.newPlainText("Public Address", selectedAddress);
        clipboardManager.setPrimaryClip(clipData);
        Toast.makeText(this, "Address Copied", Toast.LENGTH_SHORT).show();
    }

    public void shareAddress(View view) {
        Intent sendIntent = new Intent();
        sendIntent.setAction(Intent.ACTION_SEND);
        sendIntent.putExtra(Intent.EXTRA_TEXT, selectedAddress);
        sendIntent.setType("text/plain");
        startActivity(sendIntent);
    }

    private void showToast(String message) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }

    private Bitmap generateQRCode(String text) {
        MultiFormatWriter multiFormatWriter = new MultiFormatWriter();
        Bitmap bitmap = Bitmap.createBitmap(10, 10, Bitmap.Config.RGB_565);
        try {
            //Text encoded to QR BitMatrix
            BitMatrix bitMatrix = multiFormatWriter.encode(text, BarcodeFormat.QR_CODE, 1000, 1000);
            BarcodeEncoder barcodeEncoder = new BarcodeEncoder();
            //QR Bitmatrix encoded to Bitmap
            bitmap = barcodeEncoder.createBitmap(bitMatrix);
        } catch (WriterException e) {
            e.printStackTrace();
        } finally {
            return bitmap;
        }
    }

}
